﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Extensions.Logging;
using TourCompanyContracts.BusinessLogicsContracts;
using TourCompanyContracts.BindingModels;

using TourCompanyContracts.SearchModels;
using TourCompanyContracts.StoragesContracts;
using TourCompanyContracts.ViewModels;

namespace TourCompanyBusinessLogic.BusinessLogics
{
    internal class ExecurtionLogic : IExecurtionLogic
    {
        private readonly ILogger _logger;
        private readonly IExecurtionStorage _execurtionStorage;
        public ExecurtionLogic(ILogger<ExecurtionLogic> logger, IExecurtionStorage execurtionStorage)
        {
            _logger = logger;
            _execurtionStorage = execurtionStorage;
        }
        public List<ExecurtionViewModel>? ReadList(ExecurtionSearchModel? model)
        {
            _logger.LogInformation("ReadList. Purpose:{Purpose}. Id:{ Id}", model?.Purpose, model?.Id);
            var list = model == null ? _execurtionStorage.GetFullList() : _execurtionStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public ExecurtionViewModel? ReadElement(ExecurtionSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Purpose:{Purpose}. Id:{ Id}", model.Purpose, model.Id);
            var element = _execurtionStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(ExecurtionBindingModel model)
        {
            CheckModel(model);
            if (_execurtionStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(ExecurtionBindingModel model)
        {
            CheckModel(model);
            if (_execurtionStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(ExecurtionBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_execurtionStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(ExecurtionBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.Purpose))
            {
                throw new ArgumentNullException("Нет цели экскурсии", nameof(model.Purpose));
            }
            _logger.LogInformation("Execurtion. Purpose:{Purpose}. Id: { Id}", model.Purpose, model.Id);
            var element = _execurtionStorage.GetElement(new ExecurtionSearchModel
            {
                Purpose = model.Purpose
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Компонент с таким названием уже есть");
            }
        }
    }
}
