﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TourCompanyContracts.BindingModels;
using TourCompanyContracts.BusinessLogicsContracts;
using TourCompanyContracts.SearchModels;
using TourCompanyContracts.StoragesContracts;
using TourCompanyContracts.ViewModels;

namespace TourCompanyBusinessLogic.BusinessLogics
{
    internal class PlaceVisitLogic : IPlaceVisitLogic
    {
        private readonly ILogger _logger;
        private readonly IPlaceVisitStorage _placeVisitStorage;
        public PlaceVisitLogic(ILogger<PlaceVisitLogic> logger, IPlaceVisitStorage placeVisitStorage)
        {
            _logger = logger;
            _placeVisitStorage = placeVisitStorage;
        }
        public List<PlaceVisitViewModel>? ReadList(PlaceVisitSearchModel? model)
        {
            _logger.LogInformation("ReadList. UserId:{UserId}. TourGroupId:{TourGroupId}. Id: { Id}", model?.UserId, model?.TourGroupId, model?.Id);
            var list = model == null ? _placeVisitStorage.GetFullList() : _placeVisitStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public PlaceVisitViewModel? ReadElement(PlaceVisitSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement.UserId:{UserId}. TourGroupId:{TourGroupId}. Id: { Id}", model.UserId, model.TourGroupId, model.Id);
            var element = _placeVisitStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(PlaceVisitBindingModel model)
        {
            CheckModel(model);
            if (_placeVisitStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(PlaceVisitBindingModel model)
        {
            CheckModel(model);
            if (_placeVisitStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(PlaceVisitBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_placeVisitStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(PlaceVisitBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.UserId == null)
            {
                throw new ArgumentNullException("нет внешнего ключа пользователя", nameof(model.UserId));
            }
            if (model.TourGroupId == null)
            {
                throw new ArgumentNullException("нет внешнего ключа тур группы", nameof(model.TourGroupId));
            }
            _logger.LogInformation("PlaceVisit. UserId:{UserId}. TourGroupId:{TourGroupId}. Id: { Id}", model.UserId, model.TourGroupId, model.Id);
            var element = _placeVisitStorage.GetElement(new PlaceVisitSearchModel
            {
                UserId = model.UserId,
                TourGroupId = model.TourGroupId

            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Компонент с таким названием уже есть");
            }
        }
    }
}
