﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TourCompanyContracts.BindingModels;
using TourCompanyContracts.BusinessLogicsContracts;
using TourCompanyContracts.SearchModels;
using TourCompanyContracts.StoragesContracts;
using TourCompanyContracts.ViewModels;

namespace TourCompanyBusinessLogic.BusinessLogics
{
    internal class TourGroupLogic : ITourGroupLogic
    {
        private readonly ILogger _logger;
        private readonly ITourGroupStorage _tourGroupStorage;
        public TourGroupLogic(ILogger<TourGroupLogic> logger, ITourGroupStorage tourGroupStorage)
        {
            _logger = logger;
            _tourGroupStorage = tourGroupStorage;
        }
        public List<TourGroupViewModel>? ReadList(TourGroupSearchModel? model)
        {
            _logger.LogInformation("ReadList. UserId:{UserId}. Id: { Id}", model?.UserId, model?.Id);
            var list = model == null ? _tourGroupStorage.GetFullList() : _tourGroupStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public TourGroupViewModel? ReadElement(TourGroupSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement.UserId:{UserId}. Id: { Id}", model.UserId, model.Id);
            var element = _tourGroupStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(TourGroupBindingModel model)
        {
            CheckModel(model);
            if (_tourGroupStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(TourGroupBindingModel model)
        {
            CheckModel(model);
            if (_tourGroupStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(TourGroupBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_tourGroupStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(TourGroupBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.UserId == null)
            {
                throw new ArgumentNullException("нет внешнего ключа пользователя", nameof(model.UserId));
            }

            _logger.LogInformation("TourGroup. UserId:{UserId}. Id: { Id}", model.UserId, model.Id);
            var element = _tourGroupStorage.GetElement(new TourGroupSearchModel
            {
                UserId = model.UserId

            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Компонент с таким названием уже есть");
            }
        }
    }
}
