﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TourCompanyContracts.BindingModels;
using TourCompanyContracts.BusinessLogicsContracts;
using TourCompanyContracts.SearchModels;
using TourCompanyContracts.StoragesContracts;
using TourCompanyContracts.ViewModels;

namespace TourCompanyBusinessLogic.BusinessLogics
{
    internal class TourLogic : ITourLogic
    {
        private readonly ILogger _logger;
        private readonly ITourStorage _tourStorage;
        public TourLogic(ILogger<TourLogic> logger, ITourStorage tourStorage)
        {
            _logger = logger;
            _tourStorage = tourStorage;
        }
        public List<TourViewModel>? ReadList(TourSearchModel? model)
        {
            _logger.LogInformation("ReadList. UserId:{UserId}. Id: { Id}", model?.UserId, model?.Id);
            var list = model == null ? _tourStorage.GetFullList() : _tourStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public TourViewModel? ReadElement(TourSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement.UserId:{UserId}. Id: { Id}", model.UserId, model.Id);
            var element = _tourStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(TourBindingModel model)
        {
            CheckModel(model);
            if (_tourStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(TourBindingModel model)
        {
            CheckModel(model);
            if (_tourStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(TourBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_tourStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(TourBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.UserId == null)
            {
                throw new ArgumentNullException("нет внешнего ключа пользователя", nameof(model.UserId));
            }

            _logger.LogInformation("Tour. UserId:{UserId}. Id: { Id}", model.UserId, model.Id);
            var element = _tourStorage.GetElement(new TourSearchModel
            {
                UserId = model.UserId

            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Компонент с таким названием уже есть");
            }
        }
    }
}
