﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TourCompanyContracts.BindingModels;
using TourCompanyContracts.BusinessLogicsContracts;
using TourCompanyContracts.SearchModels;
using TourCompanyContracts.StoragesContracts;
using TourCompanyContracts.ViewModels;

namespace TourCompanyBusinessLogic.BusinessLogics
{
    internal class TripLogic : ITripLogic
    {
        private readonly ILogger _logger;
        private readonly ITripStorage _tripStorage;
        public TripLogic(ILogger<TripLogic> logger, ITripStorage tripStorage)
        {
            _logger = logger;
            _tripStorage = tripStorage;
        }
        public List<TripViewModel>? ReadList(TripSearchModel? model)
        {
            _logger.LogInformation("ReadList. UserId:{UserId}. Id: { Id}", model?.UserId, model?.Id);
            var list = model == null ? _tripStorage.GetFullList() : _tripStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public TripViewModel? ReadElement(TripSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement.UserId:{UserId}. Id: { Id}", model.UserId, model.Id);
            var element = _tripStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(TripBindingModel model)
        {
            CheckModel(model);
            if (_tripStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(TripBindingModel model)
        {
            CheckModel(model);
            if (_tripStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(TripBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_tripStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(TripBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.UserId == null)
            {
                throw new ArgumentNullException("нет внешнего ключа пользователя", nameof(model.UserId));
            }

            _logger.LogInformation("Trip. UserId:{UserId}. Id: { Id}", model.UserId, model.Id);
            var element = _tripStorage.GetElement(new TripSearchModel
            {
                UserId = model.UserId

            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Компонент с таким названием уже есть");
            }
        }
    }
}
